from importlib.metadata import version, PackageNotFoundError

def help():
    return """
Tonviewer -[OPTIONS] "[FOR-OPTION]"

Options
    -b, --balance                   Prints balance of wallet .
    -a, --about                     Prints description of the coin .
    -p, --price                     Prints live price #ex: toncoin , bitcoin , ... .
    -t, --transactions              Get transactions of Wallet . 
    -l, --limit                     Number of times to get transactions .
    -h, --help                      Display help message .
    -v, --version                   Show Version Of Libarary and info .
"""

def versions():
    try:
        pkg_version = version("Tonviewer")
    except PackageNotFoundError:
        pkg_version = "unknown"

    return f"""
SyncAi Version: {pkg_version} .
Author: PyCodz Channel .
(PyPi) : https://pypi.org/project/Tonviewer .
(Telegram) : https://t.me/PyCodz .
(My Portfolio) : https://deep.is-a.dev .
"""


import argparse
import asyncio
from .Scraping import Scraping
from .main import help , versions

def parse_args() -> argparse.Namespace:
    parser = argparse.ArgumentParser(
        description="TON Crypto Info Scraper is a Python library that allows you to fetch ( real-time balance of wallet , transactions , price ) data without needing any APIs.",
        add_help=False
    )
    parser.add_argument("-b", "--balance", type=str, help=" Prints balance of wallet")
    parser.add_argument("-p", "--price", type=str, help="Prints live price #ex: toncoin , bitcoin , ...")
    parser.add_argument("-a", "--about", type=str, help="Prints description of the coin")
    parser.add_argument("-t" , "--transactions" , type=str, help= "get transactions of Wallet")
    parser.add_argument("-l", "--limit", type=int, help="Number of times to get transactions")
    parser.add_argument("-h", "--help", action="store_true", help="Display help message")
    parser.add_argument("-v", "--version", action="store_true", help="Display version info")


    args, unknown = parser.parse_known_args()

    if unknown:
        for arg in unknown:
            print(f"[ERROR] Argument `{arg}` not recognized.")
        exit(1)

    return args


async def run_cli():
    args = parse_args()

    try:
        if args.balance:
            print(Scraping().get_tonviewer(args.balance))

        elif args.price:
            print(Scraping().get_coingecko_price(args.price))

        elif args.about:
            Scraping().getCoingeckoAbout(args.about)

        elif args.transactions:
            print(Scraping().get_transactions(args.transactions , args.limit))

        elif args.help:
            print(help())

        elif args.version:
            print(versions())

        else:
            print("[ERROR] No valid arguments provided. Use `-h` for help.")
    except Exception as e:
        print(f"[FATAL] Unexpected error: {e}")


def console():
    asyncio.run(run_cli())


if __name__ == "__main__":
    console()



from .Scraping import Scraping

class Wallet:
    def __init__(self, wallet: str):
        self.wallet = wallet
        self.scraper = Scraping()

    def balance(self):
        print(self.scraper.get_tonviewer(self.wallet))

    def transactions(self , limit: int=1):
        print(self.scraper.get_transactions(self.wallet , limit))








import cloudscraper
from bs4 import BeautifulSoup
import json
import re

class Scraping:
    def __init__(self):
        self.scraper = cloudscraper.create_scraper()

    def get_tonviewer(self, wallet: str):
        url = f"https://tonviewer.com/{wallet}"
        response = self.scraper.get(url)

        if response.status_code == 200:
            soup = BeautifulSoup(response.text, "html.parser")
            ton = soup.find("div", {"class": "bdtytpm b1249k0b"})
            usdt = soup.find("div", class_="bdtytpm b1ai646e")
            if ton and usdt:
                return f"{ton.text.strip() + usdt.text.strip()}"
        return None

    def get_coingecko_price(self, coin: str):
        response = self.scraper.get(f"https://www.coingecko.com/en/coins/{coin}")
        soup = BeautifulSoup(response.text, "html.parser")
        if response.status_code == 200:
            try:
                price_div = soup.find("div",
                                      class_="tw-font-bold tw-text-gray-900 dark:tw-text-moon-50 tw-text-3xl md:tw-text-4xl tw-leading-10")
                if price_div:
                    span = price_div.find("span", attrs={"data-converter-target": "price"})
                    if span:
                        return span.text.strip()
            except Exception as e:
                return " Error fetching price:", e
            return None

    def getCoingeckoAbout(self , coin):
        soup = BeautifulSoup(self.scraper.get(f"https://www.coingecko.com/en/coins/{coin}"), 'html.parser')
        try:
            about_section = soup.find('div', class_="coin-page-editorial-content")
            if about_section:
                first_paragraph = about_section.find('p')
                if first_paragraph:
                    print(first_paragraph.text.strip())
                else:
                    print("❌ not found.")
            else:
                print("❌ About section not found.")
        except Exception as e:
            print("❌ Error extracting about text:", e)


    def get_transactions(self , wallet : str , limit: int):
        global usdt , doller
        url = f"https://tonviewer.com/{wallet}"
        response = self.scraper.get(url)
        soup = BeautifulSoup(response.text, "html.parser")

        transactions = soup.find_all("a", class_="eyh3sfr")
        data = []

        for i, tx in enumerate(transactions[:limit], start=1):

            time_tag = tx.select_one("div.bdtytpm")
            time = time_tag.get_text(strip=True) if time_tag else ""

            action_tag = tx.select_one("div.text")
            action = action_tag.get_text(strip=True) if action_tag else ""


            address_tag = tx.select_one("div.action-address")
            to = address_tag.get_text(" ", strip=True) if address_tag else ""

            payload_tag = tx.select_one("div[class*=payload]")
            if payload_tag:
                text_full = payload_tag.get_text(" ", strip=True)
                text_clean = text_full.split("#")[0].strip()
                match = re.match(r"^\d+\s+\w+(?:\s+\w+)?", text_clean)
                for_what = match.group(0) if match else text_clean
            else:
                for_what = ""
            ton_tag = tx.select_one("div[class*=transfer]")
            ton = ton_tag.get_text(strip=True) if ton_tag else ""
            abss = re.search(r"[+-]?\d+(\.\d+)?" , ton)
            if abss:
                usdt = abs(float(abss.group()))
                doller =usdt * float(self.get_coingecko_price('toncoin').split("$")[1])


            data.append({
                "Time": time,
                "Action": action,
                "From" : wallet,
                "To": to,
                "Paid For": for_what,
                "Price": f"{ton} ≈ {doller} $",
                "Limit": str(i)
            })

        return json.dumps(data, indent=2, ensure_ascii=False)



















from .Scraping import Scraping

class Coin:
    def __init__(self, coin: str):
        self.coin = coin
        self.scraper = Scraping()

    def price(self):
        print(self.scraper.get_coingecko_price(self.coin))

    def about(self):
        print(self.scraper.getCoingeckoAbout(self.coin))



            




